#include "../include/Crypt.h"

Crypt::Crypt(){

}

/** simple
 @brief Simple Crypt replacement encrypted. It will replace each character from a sentances with the character at a defined distance in alphabetical.
 @see replaceChar
 @parameter string sentances, the sentance to encrypted
 @parameter key, the distance to shift
 @return string, the encrypted sentance
 **/
std::string Crypt::cesar( std::string sentances, int key ){
  std::string result = "";

  for( char character : sentances ){
    result += Crypt::replaceChar( character, key ) ;
  }
  
  return result;
}

/** vigenere
 @brief A lot mof stronger than the previous, even it's still a weak encryption algorythm. This is the same principle, however the key is modified for each character. The most elegant way to defined this dequance is with another sentance. Each character of the key sentance will defined the "a", or the key if cast in an integer
 @see loopInString
 @see replaceChar
 @see alphabet
 @return string, the vigenere encrypted sentance
 **/
std::string Crypt::vigenere( std::string sentance, std::string key ){
  std::string result = "";
  int k = 0;
  int count = 1;

  for( char character : sentance ){
    k = alphabetIndex( loopInString( key, count++ ));
    result += Crypt::replaceChar( character, k ) ;
  }
  
  return result;
}

/** replaceChar
 @brief replace a character with the next character at key distance in the alphabetical order, and loop at z.
 @parameter char ch, the chasacter to replace
 @parameter int key, the distance to look at
 @return string, the replacement character
 **/
char Crypt::replaceChar( char ch, int key ){
  ch = (char)std::tolower( ch );
  char a = 'a';
  char z = 'z';
  int max = (int)z;
  int min = (int)a;

  if(( (int)ch < min ) || ( ch > max )){
    return ch;
  }
  
  if( (ch + key) > max ){
    key -= (max - (int)ch);
    ch = 'a';
  }

  return (char)((int)ch + key);
}

/** loopInString
 @brief return a character of the sentance, defined by the integer and the character place in the sentance. However, if the integer is longer than the sentance, it will loop back at the beginning.
 @return char
 **/
char Crypt::loopInString( std::string str, int key ){
  key -= ((int)str.length() < key )? str.length() : 0 ;

  return str.c_str()[ key - 1 ];
}

/** alphabetIndex
 @brief return the number of the character in the alphabet order. Capital letter are lowered, other character are return untouched.
 @paramater char ch, the character to look at
 @return int, ordered index in the alphabet
 **/
int Crypt::alphabetIndex( char ch ){
  ch = (char)std::tolower( ch );

  char a = 'a';
  char z = 'z';
  int max = (int)z;
  int min = (int)a;

  if(( (int)ch < min ) || ( (int)ch > max )){
    return ch;
  }
  
  return int(ch) - min + 1;
}
